%{?scl:%scl_package lld}
%{!?scl:%global pkg_name %{name}}

#%%global rc_ver 3
%global baserelease 4
%global lld_srcdir lld-%{version}%{?rc_ver:rc%{rc_ver}}.src
%global maj_ver 9

# Don't include unittests in automatic generation of provides or requires.
%global __provides_exclude_from ^%{_libdir}/lld/.*$
%global __requires_exclude ^libgtest.*$

%bcond_with ld_alternative

%ifnarch i686
%global enable_test_pkg 1
%endif

Name:		%{?scl_prefix}lld
Version:	%{maj_ver}.0.1
Release:	%{baserelease}%{?rc_ver:.rc%{rc_ver}}%{?dist}
Summary:	The LLVM Linker

License:	NCSA
URL:		http://llvm.org
Source0:	http://%{?rc_ver:pre}releases.llvm.org/%{version}/%{?rc_ver:rc%{rc_ver}}/%{lld_srcdir}.tar.xz
Source1:	run-lit-tests
Source2:	lit.lld-test.cfg.py

Patch0:		0001-CMake-Check-for-gtest-headers-even-if-lit.py-is-not-.patch

# RHEL Patches:
# The test cases in this patch crash with:
# terminate called after throwing an instance of 'std::regex_error
# this seems like it might be an issue with older libstdc++, since we
# don't need this feature in RHEL, just revert the patch.
Patch100:	0001-Revert-lld-Add-Visual-Studio-compatible-diagnostics.patch

BuildRequires:	%{?scl_prefix}cmake
BuildRequires: %{?scl_prefix}llvm-devel = %{version}
%if 0%{?enable_test_pkg}
BuildRequires:	%{?scl_prefix}llvm-test = %{version}
%endif
BuildRequires:	ncurses-devel
BuildRequires:	zlib-devel
BuildRequires:	chrpath
BuildRequires:	python2-devel

# For make check:
BuildRequires: python
BuildRequires: %{?scl_prefix}python-lit = 0.8.0
BuildRequires: %{?scl_prefix}llvm-googletest

Requires: %{?scl_prefix}lld-libs = %{version}-%{release}

%description
The LLVM project linker.

%package devel
Summary:	Libraries and header files for LLD

Requires: %{?scl_prefix}lld-libs = %{version}-%{release}

%description devel
This package contains library and header files needed to develop new native
programs that use the LLD infrastructure.

%package libs
Summary:	LLD shared libraries

%description libs
Shared libraries for LLD.

%package test
Summary: LLD regression tests
Requires:      %{name}%{?_isa} = %{version}-%{release}
Requires:      %{?scl_prefix}python-lit
Requires:      %{?scl_prefix}llvm-test(major) = %{maj_ver}
Requires:      %{?scl_prefix}lld-libs = %{version}-%{release}

%description test
LLVM regression tests.

# LLVM libraries and tools no longer set rpath, so we need to enable the scl
# so llvm-config can find libLLVM.so.
%enable_llvmtoolset9_0

%prep
%autosetup -n %{pkg_name}-%{version}%{?rc_ver:rc%{rc_ver}}.src -p2

%build

mkdir %{_target_platform}
cd %{_target_platform}

%cmake .. \
	-DLLVM_TEMPORARILY_ALLOW_OLD_TOOLCHAIN:BOOL=ON \
	-DLLVM_LINK_LLVM_DYLIB:BOOL=ON \
	-DLLVM_DYLIB_COMPONENTS="all" \
	-DPYTHON_EXECUTABLE=%{__python2} \
	-DLLVM_INCLUDE_TESTS=ON \
	-DLLVM_MAIN_SRC_DIR=%{_datadir}/llvm/src \
	-DLLVM_EXTERNAL_LIT=%{_bindir}/lit \
	-DLLVM_LIT_ARGS="-sv \
	--path %{_libdir}/llvm" \
%if 0%{?__isa_bits} == 64
	-DLLVM_LIBDIR_SUFFIX=64
%else
	-DLLVM_LIBDIR_SUFFIX=
%endif

%make_build

# Build the unittests so we can install them.
%make_build lld-test-depends

%install

%global lit_cfg test/%{_arch}.site.cfg.py
%global lit_unit_cfg test/Unit/%{_arch}.site.cfg.py
%global lit_lld_test_cfg_install_path %{_datadir}/lld/lit.lld-test.cfg.py

# Generate lit config files.  Strip off the last line that initiates the
# test run, so we can customize the configuration.
head -n -1 %{_target_platform}/test/lit.site.cfg.py >> %{lit_cfg}
head -n -1 %{_target_platform}/test/Unit/lit.site.cfg.py >> %{lit_unit_cfg}

# Patch lit config files to load custom config:
for f in %{lit_cfg} %{lit_unit_cfg}; do
  echo "lit_config.load_config(config, '%{lit_lld_test_cfg_install_path}')" >> $f
done

# Install test files
install -d %{buildroot}%{_datadir}/lld/src
cp %{SOURCE2} %{buildroot}%{_datadir}/lld/

tar -czf %{buildroot}%{_datadir}/lld/src/test.tar.gz test/
install -d %{buildroot}%{_libexecdir}/tests/lld
cp %{SOURCE1} %{buildroot}%{_libexecdir}/tests/lld

# Install unit test binaries
install -d %{buildroot}%{_libdir}/lld/
cp -R %{_target_platform}/unittests %{buildroot}%{_libdir}/lld/
rm -rf `find %{buildroot}%{_libdir}/lld/ -iname '*make*'`

# Install gtest libraries
cp %{_target_platform}/%{_lib}/libgtest*so* %{buildroot}%{_libdir}/lld/

# Install libraries and binaries
cd %{_target_platform}
%make_install

# Remove rpath
chrpath --delete %{buildroot}%{_bindir}/*
chrpath --delete %{buildroot}%{_libdir}/*.so*
chrpath --delete `find %{buildroot}%{_libdir}/lld/ -type f`

%if %{with ld_alternative}
# Required when using update-alternatives:
# https://docs.fedoraproject.org/en-US/packaging-guidelines/Alternatives/
touch %{buildroot}%{_bindir}/ld

%post
%{_sbindir}/update-alternatives --install %{_bindir}/ld ld %{_bindir}/ld.lld 1

%postun
if [ $1 -eq 0 ] ; then
  %{_sbindir}/update-alternatives --remove ld %{_bindir}/ld.lld
fi
%endif

%check

%if 0%{?enable_test_pkg}
make -C %{_target_platform} %{?_smp_mflags} check-lld
%endif

%post libs -p /sbin/ldconfig
%postun libs -p /sbin/ldconfig

%files
%{_bindir}/lld*
%{_bindir}/ld.lld
%{_bindir}/ld64.lld
%{_bindir}/wasm-ld

%files devel
%{_includedir}/lld
%{_libdir}/liblld*.so

%files libs
%{_libdir}/liblld*.so.*

%files test
%{_libexecdir}/tests/lld/
%{_libdir}/lld/
%{_datadir}/lld/src/test.tar.gz
%{_datadir}/lld/lit.lld-test.cfg.py
# FIXME: The build system auto compiles the python bytecode and I'm not
# sure how to turn it off.
%{_datadir}/lld/lit.lld-test.cfg.pyc
%{_datadir}/lld/lit.lld-test.cfg.pyo

%changelog
* Tue Mar 10 2020 Tom Stellard <tstellar@redhat.com> - 9.0.1-4
- Depend on python2-lit.  There is no python3 in rhel7

* Tue Mar 10 2020 Tom Stellard <tstellar@redhat.com> - 9.0.1-3
- Fix dependency on python3-lit

* Fri Jan 31 2020 Tom Stellard <tstellar@redhat.com> - 9.0.1-2
- Add scl_prefix to lld-test requires

* Tue Jan 28 2020 Tom Stellard <tstellar@redhat.com> - 9.0.1.-1
- 9.0.1 Release

* Wed Aug 28 2019 sguelton@redhat.com - 8.0.1-5
- Remove __cmake hack

* Tue Aug 06 2019 Tom Stellard <tstellar@redhat.com> - 8.0.1-4
- Fix explicit -libs requires in -devel and also add it to the main package

* Tue Aug 06 2019 Tom Stellard <tstellar@redhat.com> - 8.0.1-3
- Add explicit -libs requires to the -devel package

* Fri Aug 2 2019 sguelton@redhat.com - 8.0.1-2
- Update cmake dependency

* Mon Jul 29 2019 sguelton@redhat.com - 8.0.1-1
- 8.0.1 Release

* Fri Jan 11 2019 Tom Stellard <tstellar@redhat.com> - 7.0.1-2
- Make lld-libs an explicit dependency of lld-devel

* Thu Jan 10 2019 Tom Stellard <tstellar@redhat.com> - 7.0.1-1
- 7.0.1 Release

* Wed Jun 27 2018 Tom Stellard <tstellar@redhat.com> - 6.0.1-1
- 6.0.1 Release

* Fri May 11 2018 Tom Stellard <tstellar@redhat.com> - 6.0.1-0.1.rc1
- 6.0.1-rc1 Release

* Thu Mar 08 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-1
- 6.0.0 Release

* Tue Feb 13 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-0.3.rc2
- 6.0.0-rc2 Release

* Thu Feb 08 2018 Fedora Release Engineering <releng@fedoraproject.org> - 6.0.0-0.2.rc1
- Rebuilt for https://fedoraproject.org/wiki/Fedora_28_Mass_Rebuild

* Thu Jan 25 2018 Tom Stellard <tstellar@redhat.com> - 6.0.0-0.1.rc1
- 6.0.0-rc1 Release

* Thu Dec 21 2017 Tom Stellard <tstellar@redhat.com> - 5.0.1-1
- 5.0.1 Release

* Mon Sep 11 2017 Tom Stellard <tstellar@redhat.com> - 5.0.0-1
- 5.0.0 Release

* Thu Aug 03 2017 Fedora Release Engineering <releng@fedoraproject.org> - 4.0.1-4
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Binutils_Mass_Rebuild

* Wed Jul 26 2017 Fedora Release Engineering <releng@fedoraproject.org> - 4.0.1-3
- Rebuilt for https://fedoraproject.org/wiki/Fedora_27_Mass_Rebuild

* Thu Jul 06 2017 Tom Stellard <tstellar@redhat.com> - 4.0.1-2
- Backport r307092

* Tue Jul 04 2017 Tom Stellard <tstellar@redhat.com> - 4.0.1-1
- 4.0.1 Release

* Tue Jul 04 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-4
- Fix build without llvm-static

* Wed May 31 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-3
- Remove llvm-static dependency

* Mon May 15 2017 Fedora Release Engineering <rel-eng@lists.fedoraproject.org> - 4.0.0-2
- Rebuilt for https://fedoraproject.org/wiki/Fedora_26_27_Mass_Rebuild

* Tue Mar 14 2017 Tom Stellard <tstellar@redhat.com> - 4.0.0-1
- lld 4.0.0 Final Release
